/**
 * Introduction
 *
 * @title ROV - Easy applications from C
 *
 * @copyright Jason Tribbeck / 7th ARM 1994-2001
 *
 * @authors Jason Tribbeck
 *
 * @file print.h
 *
 * @type subsystem
 *
 * @description
 *
 * The print subsystem is used for interfacing to printers.
 */


#ifndef __rov_print
#define __rov_print


#ifdef __cplusplus
extern "C"
{
#endif

#ifndef __rov_main
#include "main.h"
#endif

#include "kernel.h"

/**
 * Definition of a page size.
 * @field x_size the width of the page.
 * @field y_size the height of the page.
 * @field left the left hand margin.
 * @field bottom the bottom margin.
 * @field right the right hand margin.
 * @field top the top margin.
 */
typedef struct {
  int x_size;
  int y_size;
  int left;
  int bottom;
  int right;
  int top;
} print_pagesizestr;

/**
 * Definition of a plot rectangle.
 * @field x0 the left hand side.
 * @field y0 the bottom.
 * @field x1 the right hand side.
 * @field y1 the top.
 */
typedef struct {
  int x0;
  int y0;
  int x1;
  int y1;
} print_plotrectangle;

/**
 * Definition of a print transformation.
 * @field a the first part of the matrix.
 * @field b the second part of the matrix.
 * @field c the third part of the matrix.
 * @field d the fourth part of the matrix.
 */
typedef struct {
  int a;
  int b;
  int c;
  int d;
} print_transform;

/**
 * Defintion of a print offset.
 * @field ox the x offset.
 * @field oy the y offset.
 */
typedef struct {
  int ox;
  int oy;
} print_offset;

/**
 * Definition of a print rectangle.
 * @param job the job number.
 * @param rect the rectangle to print.
 */
typedef void (* print_update)(int job,
			      print_plotrectangle *rect);

/** 
 * Get the name of the printer.
 * @return the name of the printer, or NULL if no printer driver loaded.
 */
char *print_getprintername(void);

/**
 * Get the printer features.
 * @return the printer features.
 */
int print_getfeatures(void);

/**
 * Get the page size
 * @param page_size the buffer to hold the page size.
 */
void print_pagesize(print_pagesizestr *page_size);

/**
 * Makes a print job the current job.
 * @param job the job to use.
 * @param text the page text. ???
 * @return non-zero for success.
 */
int print_selectjob(int job,
		    char * text);

/**
 * Get the current print job.
 * @return the current print job.
 */
int print_currentjob(void);

/**
 * End a print job.
 * @param job the job to end.
 */
void print_endjob(int job);

/**
 * Abort a print job.
 * @param job the job to abort.
 */
void print_abortjob(int job);

/**
 * Give a print rectangle to a job.
 * @param job the job.
 * @param rect the print rectangle.
 * @param xform the print transform.
 * @param offset the print offset.
 * @param page the page number. ???
 */
void print_giverectangle(int job,
			 print_plotrectangle *rect,
			 print_transform *xform,
			 print_offset *offset,
			 int page);

/**
 * Start printing.
 * @param job the job to print.
 * @param page the page to print. ???
 * @param title the job title.
 * @param update the print update handler.
 * @return an error, or NULL if no error has occurred.
 */
_kernel_oserror * print_print(int job,
			      int page,
			      char *title,
			      print_update update);

/**
 * Cancel a job.
 * @param job the job to cancel.
 * @see print_canceljobwitherror()
 */
void print_canceljob(int job);

/**
 * Cancel a job with an error.
 * @param job the job to cancel.
 * @see print_canceljob()
 */
void print_canceljobwitherror(int job);

/**
 * Declare a font before printing.
 * @param job the print job.
 * @param font the font name.
 * @param size the font size.???
 */
void print_declarefont(int job,
		       char *font,
		       int size);

/**
 * Open a file.
 * @param file_name the name of the file.
 * @return the file handle.
 * @see print_closefile()
 */
int print_openfile(char * file_name);

/**
 * Close a file.
 * @param handle the file handle.
 * @see print_openfile()
 */
void print_closefile(int handle);


#ifdef __cplusplus
}
#endif

#endif
